include "Generic_types.thrift"
include "Factory_types.thrift"
include "File_types.thrift"
include "Common_factory_commands.thrift"

namespace cpp Distant_cmd
namespace java com.morpho.distant_cmd
 
service CIM_factory_commands extends Common_factory_commands.Common_factory_commands
{
    /**
     * Get MEP version
     *
     * Return string containing MEP version	 
     */
	string factory_get_MEP_version()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept);

    /**
     * Upgrade customer key
     *
     * This function is used to upgrade the customer key on the terminal that is used to validate the signature 
     * of the third party applications.
     *
     * <code>Generic_commands.file_load</code> command should be called prior to this command for loading the key upgrade 
     * package on the terminal.
     *
     * Return 0: success, 1: failure
     */
    bool upgrade_customer_key()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
                    /** Signature failure*/
                    2: Factory_types.Fac_firmware_upgrade_signature_failure_error signature_failure_xcept
                    /** File not loaded */
                    3: Factory_types.Fac_file_not_loaded_error file_not_loaded_xcept );
    
    /**
     * Upgrade customer application
     *
     * This function is used to upgrade the customer application on the terminal (partial retrofit).
     *
     * <code>Generic_commands.file_load</code> command should be called prior to this command for loading the application upgrade 
     * package on the terminal.
     *
     * Return 0: success, 1: failure
     */
    bool upgrade_customer_application()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept
                    /** Signature failure*/
                    2: Factory_types.Fac_firmware_upgrade_signature_failure_error signature_failure_xcept
                    /** File not loaded */
                    3: Factory_types.Fac_file_not_loaded_error file_not_loaded_xcept );
				
	/**
     * Get barcode reader Firmware version
     *
     * This function is used to read the barcode reader Firmware version.
     *     
     * Return string containing barcode reader Firmware version
     */
    string get_bcr_firmware_version()
        throws (    /** Generic exception */
                    1: Generic_types.Generic_error generic_xcept                
                    /** Device not mounted */
                    2: Factory_types.Fac_device_unmounted_error bcr_not_mounted_xcept );
				
	/**
     * Load and upgrade barcode reader Firmware.<br>
     *
     * <b>Note:</b>
     * <ol>
     *   <li>The <code>File_types.File_chunk.action</code> shall be taken into consideration only for the last chunk,
     *     i.e., when <code>File_types.File_chunk.is_last</code> is true.
     *   <li>Maximum file size: 5 MB     
     *   <li>Maximum chunk size is 10240 bytes, i.e. 10 KB
     * </ol><br>
     * <b>Warning:</b><br>
     * <ol>
     *   <li>You shall need a modified version of the generated code that supports
     *       handling of several request/response messages for the single call to the command.
     * </ol><br>
     * <b>Functionality change required in Thrift generated client code (C++ sample)</b><br>
     * <pre>
     * void Factory_commandsClient::upgrade_bcr_firmware(const  ::Distant_cmd::File_chunk& chunk)
     * {
     *     <span style="color:green">// Original code section in the generated code
     *     // send_upgrade_bcr_firmware(chunk);
     *     // recv_upgrade_bcr_firmware();</span>
     * &nbsp;
     *     <span style="color:green">// BEGIN: Custom section for receiving and processing intermediate data
     *     // This is actually multi-request handling</span>
     *     ::Distant_cmd::File_chunk actual_chunk; <span style="color:green">// a non-const chunk required</span>
     *     while(true)
     *     {
     *         <span style="color:green">// Client's call-back is supposed to load the data that shall be available in the 'actual_chunk' parameter to send to the Thrift server.
     *         //  Note that in case of the last chunk of data to be uploaded the 'actual_chunk.is_last' flag must be set.</span>
     *         Multiresponse_handler::client_cb_upgrade_bcr_firmware(actual_chunk);
     *         send_upgrade_bcr_firmware(actual_chunk);
     * &nbsp;
     *         recv_upgrade_bcr_firmware(); <span style="color:green">// required to see if exception was received -- so that further sending is stopped</span>
     * &nbsp;
     *         if(actual_chunk.is_last) <span style="color:green">// Exit the loop since the last chunk has been sent and its corresponding response received</span>
     *         {
     *             break;
     *         }
     *     }
     *     <span style="color:green">// END: Custom section for receiving and processing intermediate data</span>
     * }
     * </pre>
     */
    void upgrade_bcr_firmware(  /** The chunk of the file */
                                1: File_types.File_chunk chunk,
                                /** md5-checksum of the BCR Firmware file */
                                2: binary checksum)
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept            
                /** The speficied file or chunk exceeds its maximum limit */
                2: Generic_types.File_too_large_error file_large_xcept,
                /** File not loaded */
                3: Factory_types.Fac_file_not_loaded_error file_not_loaded_xcept,
                /** Device not mounted */
                4: Factory_types.Fac_device_unmounted_error bcr_not_mounted_xcept
                /** BCR integrity check error  */
                5: Generic_types.fw_integrity_error fw_integrity_error_xcept);

    /**
     * Load and upgrade Printer Firmware.
     *	
     * Binary data are put as they are
     *
     * Note: This command is only supported in S4 product.
     */
     void upgrade_printer_firmware( /** binary file to be upgraded */
                                    1: binary file_data,
                                    /** md5-checksum of the printer Firmware file */
                                    2: binary checksum)
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept
                /** Printer error */
                2: Generic_types.Printer_general_error printer_error_xcept
                /** Printer integrity check error */
                3: Generic_types.fw_integrity_error fw_integrity_error_xcept);

    /**
     * Erase third party application.
     *
     * Note: This command erases all the customer application files and data. On bootup default GUI application shall start running.
     *
     */
     void erase_customer_application(   /** Flag to set if customer application key also need to be erased */
                                        1: bool erase_customer_key)
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

    /**
     * Restore default application key.
     *
     * Note: This command restore default application key.
     *
     */
     void restore_default_application_key()
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);

     /**
     * Select the retrofit components.
     *
     * Note: This command select the retrofit components for which retrofit needs to be performed. 
     * According to the selection, it will set appropriate parameter values in parameter zone and 
     * it will reboot the terminal according to boot-flag(or component selected).
     * 
     */      
     void select_retrofit_components(   /** list of selected components */
                                        1: list <Factory_types.Component_type> component_list)
        throws( /** Generic exception */
                1: Generic_types.Generic_error generic_xcept);
}