// <doc_history>
/**
 * @section LICENSE
 * Copyright &copy; 2015 Safran Morpho - Safran Morpho confidential - all rights reserved
 *
 * @section DESCRIPTION
 *
 *
 * @file Sensor_types.thrift
 *
 * CHANGELOG
 * 26 Jan 2015  - First draft created
 * 27 Jan 2015  - Multi response help added, reboot command added and comment changes 
 * 30 Mar 2015  - MPH review comments added
 *                added enumeration for "Biometric_type", ranamed Bio_template_type to Bio_template_format
 *                all fields in Bio_sensor_image are optional except data
 * 31 Mar 2015  - MPH review comments, replaced enumeration Bio_enroll_security_level by integer
 * 01 Apr 2015  - Bio_command struct is replace by enumeration Sensor_Biofinger_event
 * 10 Apr 2015  - remove "temperature_mask" from Sensor_Biofinger_event
 * 21 Apr 2015  - added notes "left and right thumb not supported" for MorphoWave product
 * 05 Aug 2015  - 'software_upgrade' file type and 'Sensor_upgrade_incompatible_version_error' exception added.
 */
// </doc_history>

namespace cpp Sensor_cmd
namespace java com.morpho.Sensor

/** Enumeration representing of the biometric operation failure.*/
enum Bio_failure_cause
{
    Bio_matching_no_hit,
    Bio_enrollment_bad_quality
}

/** Enumeration representing the events that can be monitored.*/
enum Bio_async_event
{
    command,
    images,
    enrollment_steps,
    captures_quality,
}

/**
 * Enumeration representing the finger position/command event sent by the sensor
 */
enum Sensor_Biofinger_event
{
    no_finger = 0,
    move_finger_up = 1,
    move_finger_down = 2,
    move_finger_left = 3,
    move_finger_right = 4,
    press_finger_harder = 5,
    latent_detected = 6,
    remove_finger = 7,
    finger_ok = 8, // same as image_captured
    finger_detected = 9, // same as presence_detected
    finger_misplaced = 10,
    live_ok = 11,
    hand_too_fast = 12,
    hand_too_inclined = 13,
    low_quality_left_index = 14,
    low_quality_left_little = 15,
    low_quality_left_middle = 16,
    low_quality_left_ring = 17,
    low_quality_left_thumb = 18,
    low_quality_right_index = 19,
    low_quality_right_little = 20,
    low_quality_right_middle = 21,
    low_quality_right_ring = 22,
    low_quality_right_thumb = 23,
    duplicate_finger = 24,
    inconsistent_fingers = 25,
    fake_finger_detected = 26,
    wrong_finger_number = 27,
    device_dirty = 28,
    glass_scratched = 29,
    capture_started = 30,
    pre_proc_done = 31,
    coding_matching_done = 32,
    temperature_high = 33,
    temperature_too_high = 34
}

/**
 * Structure for callback events
 */
struct Bio_callback_event
{
    /**
    * Callback events regarding placement modification required, or data acquisition completed.</br>
    * Sent as intermediate reply of biometric event.
    */
    1: optional list < Sensor_Biofinger_event > event;
}

/** Enumeration representing compression used in an exported image.*/
enum Bio_compression_type
{
    null_algo,
    v1_algo,
    wsq_algo,
}

/**
 * Structure for sensor image
 */
struct Bio_sensor_image
{
    /** Width of the image in pixels.*/
    1: optional i32 width_in_pix,
    /** Height of the image in pixels.*/
    2: optional i32 height_in_pix,
    /** Horizontal resolution of the image in dot per inch.*/
    3: optional i32 horizontal_resol_dpi,
    /** Vertical resolution of the image in dot per inch.*/
    4: optional i32 vertical_resol_dpi,
    /** Compression algorithm used on the image.*/
    5: optional Bio_compression_type  compression_type,
    /** Image's binary data.*/
    6: required binary image_data,
    /**
     * Number of bits per pixel for the image.</br>
     * Present in intermediate replies' images of biometric commands.
     */
    7: optional byte bits_per_pixel,
    /** image identifier. 
	 * Number that represents the enrolled image identifier number </br>
	 * i.e. for finger biometric data, this number represents finger enrolled. </br>
	 * 1 for left little finger, up to 10 for right little finger. </br>
	 * <note> For MorphoWave sensor left thumb and right thumb are not supported. </note>     	 
	 */
    8: optional byte   image_identifier;		
}


/** Enumeration representing the format handled by the sensor in biometric operations.*/
enum Bio_template_format
{
	/** ISO/IEC 19794-2:2005 Finger Minutiae Card Format Compact Size (aka. ISO FMC CS).*/
	ISO_19794_2_CF_CS,
	
	/** ISO/IEC 19794-2:2005 Finger Minutiae Card Format Normal Size (aka. ISO FMC NS).*/
	ISO_19794_2_CF_NS,
	
	/** ISO/IEC 19794-2:2005 Finger Minutiae Record (aka. ISO FMR).*/
	ISO_19794_2,
	
	/** ANSI INCITS 378-2004 Finger Minutiae Record (aka. ANSI FMR).*/
	ANSI_378,
	
	/** Morpho proprietary PkLite Fingerprint Template.*/
	pklite, 
	
	/** Morpho legacy PkComp V2 Fingerprint Template (compressed, from 135 to 256 bytes; use for input only).*/
	pkcompv2,
	
	/** Morpho legacy PkMat Fingerprint Template (uncompressed, 512 bytes fixed; use for input only).*/
	pkmat	
}

/**
 * Enumeration representing for Biometric_type
 */
enum Biometric_type 
{
    fingerprint,
}

/**
 * Structure for Biometric record 
 */
struct Bio_record
{
    /** Type of the template.*/
    1: optional Bio_template_format template_format;
    /** Template data.*/
    2: optional binary   template_data;
    /** Template number. 
	 * Number that represents the enrolled template identifier number </br>
	 * i.e. for finger biometric data, this number represents finger enrolled. </br>
	 * 1 for left little finger, up to 10 for right little finger. </br>
	 * <note> For MorphoWave left thumb and right thumb are not supported. </note>     	 
	 */
    3: optional byte   template_number;	
	/** Template quality.*/
    4: optional i16   template_quality;		
    
    /** Biometric type, finger print as of now*/
    5: optional Biometric_type template_biometric_type;
}

/**
 * Structure for user control parameter
 */
struct Control_param
{
	/**
	 * Max. duration of user control process<br>
	 * 0 implies an infinite process. Allowed range is 0 - 65535 seconds
	 */
    1: optional i32 timeout_in_sec,

	/**
	 * If set to false, you will receive only one reply containing the final result of the
	 * enrollment, otherwise you may also receive asynchronous replies containing the
	 * progress status of the enrollment.
	 */
	2: optional bool enable_intermediate_replies,	
	
    /** Matching score is required in final answer.*/
    3: optional bool require_matching_score,

    /** This parameter specifies the value of the False Acceptance Ratio (FAR) of the MorphoSmart\99 device.
       * The value of this parameter can be set from 0 to 10, by 1 value steps. .*/
    4: optional i32 matching_threshold,	
	
	/**
	 * User ID for local db authentication. </br> 
	 * Not used for identification process.
	 */
	5: optional string user_id_UTF8,	

	/**
	 * Reference user records for authentication against provided records, not required if user_id_UTF8 is present. </br>
	 * Not used for identification process.
	 */
	6: optional list<Bio_record> ref_record_list,
	
    /** List of event to subscribe to.*/
    7: optional list<Bio_async_event>         events,	
}

/**
 * Structure for control intermediate result
 */
struct Control_intermediate_result
{
    /**
     * Intermediate reply that is used to:
     * <ul>
     * <li>indicate that there is a wrongly placed finger on the sensor, and then the action that the user has to perform.</li>
     * <li>provide information about the state of the fingerprint process (fingerprint wait started, fingerprint acquisition completed).</li>
     * </ul>
     * It shall have been requested in the event.
     */
    1: optional Bio_callback_event          cb_bio_event,
    /**
     * Intermediate reply that is used to transmit list of images: the images of the current capture (May be several capture per enrollment).</br>
     * It shall have been requested in the command (see @a Bio_async_event::images).
     */
    2: optional list<Bio_sensor_image>              cb_capture_images,
    /**
     * Intermediate reply that is used to provide the quality value of the capture (May be several capture per finger).</br>
     * It shall have been requested in the command (see @a Bio_async_event::captures_quality).
     */
    3: optional i32                           cb_capture_quality,	
}


/**
 * Structure for user control final result
 */
struct Control_final_result
{
    /**
     * Indicate enrollment result.</br>
     * True for success, false for error.
     */
    1: required bool		success,
	
    /**
     * Indicate the cause of the error.</br>
     * Present if success field is false.
     */
    2: optional Bio_failure_cause         cause,
	
    /**
     * If success field if true, score of the matching operation. </br>
     * It shall have been requested in the request.
     */
    3: optional i32 matching_score,
	
    /** If success field is true, number of the finger that matched with captured one (see Control_param::require_matching_score).*/
    4: optional byte matching_template_number,
	
    /**
     * User ID. </br>
     * Available in case of identify success result.
     */	 
    5: optional string user_ID_UTF8
}

/**
 * Structure for user control reply
 */
struct Control_reply
{
    /**
     * Final reply of user control command. </br>
     * If final_result is present, none of the other fields shall be present.
     */
    1: optional Control_final_result        final_result,
    /**
     * Intermediate reply 
     * It shall have been requested in the command (see @a Control_param::enable_intermediate_replies).
     * If intermediate_result is present, none of the other fields shall be present.
     */
    2: optional Control_intermediate_result          cb_control_intermediate_result
}


/**
 * Structure for Enrollment parameter
 */
struct Enroll_param
{
	/**
	 * Max. duration of enrollment process<br>
	 * 0 implies an infinite process. Allowed range is 0 - 65535 seconds
	 */
    1: optional i32 timeout_in_sec,

    /**
     * Biometric record of the user to enroll.</br>
     * Only parameter required here is number of records (i.e. size of list), template_format and template_number for each record to enroll 
     */
    2: optional list<Bio_record>                    bio_records,	

    /** indicate the enrollment security level to use for the operation.
     *   
     *  0 = Disable (for all sensors)
     *  For MorphoWave:
     *      1 = low
     *      2 = standard
     *      3 = secure
     *      4 = very secure 
     */
    3: optional i32   enroll_security_level,	
	
	/** Best image of each captured enrollment is required in the reply.*/
    4: optional bool	require_best_images,	
	
	/**
	 * If set to false, you will receive only one reply containing the final result of the
	 * enrollment, otherwise you may also receive asynchronous replies containing the
	 * progress status of the enrollment.
	 */
	5: optional bool enable_intermediate_replies, 

    /** List of event to subscribe to.*/
    6: optional list<Bio_async_event>         events,	
}

/**
 * Structure for Enrollment final result
 */
struct Enroll_final_result
{
    /**
     * Indicate enrollment result.</br>
     * True for success, false for error.
     */
    1: required bool		success,
    /**
     * Indicate the cause of the error.</br>
     * Present if success field is false.
     */
    2: optional Bio_failure_cause         cause,
    /**
     * Biometric record of the user.</br>
     * Present if success is true.
     */
    3: optional list<Bio_record>                    bio_records,
	
    /**
     * Binary result best images of the enrollment.</br>
     * Present if success is true and images are required in the command (see @a Enroll_param::require_best_images).</br>
     * 1 image per one enrollment.
     */
    4: optional list<Bio_sensor_image>    best_images,
}

/**
 * Structure for Enrollment steps
 */

struct Bio_callback_enrollment_step
{
    /** Current enrollment number for the enrollment process (starting to 1).*/
    1: required byte current_enroll,
    /** Total number of enrollment for the enrollment process.*/
    2: required byte total_enroll,
    /** Current capture of the current enrollment (starting to 1).*/
    3: required byte current_capture,
    /** Total number of captures for one enrollment.*/
    4: required byte total_capture
}


/**
 * Structure for Enrollment intermediate result
 */
struct Enroll_intermediate_result
{
    /**
     * Intermediate reply that is used to indicate the enrollment process steps.</br>
     * For example, during the enrollment of one finger with three images, the terminal sends three messages (enrollment of finger 1 of 1, acquisition 1, 2 and 3).</br>
     * It shall have been requested in the command (see @a Bio_async_event::enrollment_steps).
     */
    1: optional Bio_callback_enrollment_step   cb_enroll_step,
    /**
     * Intermediate reply that is used to:
     * <ul>
     * <li>indicate that there is a wrongly placed finger on the sensor, and then the action that the user has to perform.</li>
     * <li>provide information about the state of the fingerprint process (fingerprint wait started, fingerprint acquisition completed).</li>
     * </ul>
     * It shall have been requested in the event.
     */
    2: optional Bio_callback_event          cb_bio_event,
    /**
     * Intermediate reply that is used to transmit list of images: the images of the current capture (May be several capture per enrollment).</br>
     * It shall have been requested in the command (see @a Bio_async_event::images).
     */
    3: optional list<Bio_sensor_image>              cb_capture_images,
    /**
     * Intermediate reply that is used to provide the quality value of the capture (May be several capture per enrollment).</br>
     * It shall have been requested in the command (see @a Bio_async_event::captures_quality).
     */
    4: optional i32                           cb_capture_quality,	
}

/**
 * Structure for Enrollment reply
 */
struct Enroll_reply
{
    /**
     * Final reply of enroll command. </br>
     * If final_result is present, none of the other fields shall be present.
     */
    1: optional Enroll_final_result        final_result,
    /**
     * Intermediate reply 
     * It shall have been requested in the command (see @a Enroll_param::enable_intermediate_replies).
     * If intermediate_result is present, none of the other fields shall be present.
     */
    2: optional Enroll_intermediate_result          cb_enroll_intermediate_result
}

/**
 * Structure for db status
 */
struct DB_status
{
    /** Maximum number of records.*/
    1: i32      capacity;
    /** Current number of records.*/
    2: i32      size;
}

/**
 * A generic datatype
 *
 *
 * It is the generic datatype that might be used, for instance, as MA5G configuration parameter.<br>
 * Note that only one of the values of interest shall be filled in.
 */
struct Variant
{
    1: optional byte byte_value,
    2: optional i16 int16_value,
    3: optional i32 int32_value,
    4: optional i64 int64_value,
    5: optional double double_value,
    6: optional bool bool_value,
    7: optional string UTF8string_value,
    8: optional binary binary_value
}


/**
 * Sensor information
 */
struct Sensor_info
{
    /** UTF8 string that represents 10 digit sensor part number of packaged product */
    1: optional string sensor_packaged_part_number_UTF8,
    /** UTF8 string that represents 14 digit sensor serial number of packaged product */
    2: optional string sensor_packaged_serial_number_UTF8,
    /** UTF8 string that represents sensor version number */
    3: optional string sensor_version_number_UTF8,	
}

/**
 *
 * Enumeration of sensor information type
 */
enum Sensor_info_type
{
    /** 10 digit number that represents sensor part number of packaged product */
    sensor_packaged_part_number,
    /** 14 digit number that represents sensor serial number of packaged product */
    sensor_packaged_serial_number,
    /** 14 digit number that represents sensor version number */
    sensor_firmware_version_number,	
}

/**
 * Structure for sensor simulation mode
 */
struct Sensor_simulation_files
{
    /**
     * raw image file path (absolute file path on terminal)
     */
    1: optional string raw_image_file_name,
}

/**
 *
 * Enumeration for file types for file operations.
 */
enum File_type
{
    diagnostic_info,
    /** sensor application software upgrade file type */
    software_upgrade,
}

/**
 *
 * Enumeration for file sub-types for file operations.
 */
enum File_subtype
{
    /** sensor application diagnostic information */
    diagnostic_info_app,
    /** sensor library diagnostic information */
    diagnostic_info_lib,	
}

struct File_details
{
    1: File_type type;
    2: optional File_subtype subtype;
    3: optional string name_UTF8;
}

struct File_chunk
{
    /** File index of the data present in this chunk */
    1: i32      index;
    /** Set to true if this chunk is the last, false otherwise */
    2: bool     is_last;
    /** File data */
    3: binary   data;
}

/**
 *
 * Sensor error codes
 */
enum Sensor_error_code
{		
	err_unknown,
    err_sensor_in_use,
    err_cancel,
    err_parameter_mismatch,
    err_device_not_found,
    err_device_time_out,
    err_power_supply_error,
    err_sensor_internal_error,
    err_sensor_database_creation_error,
    err_invalid_template,
    err_command_not_available,
    err_user_record_not_available,
    err_user_exist,    
    err_license_missing,    
    err_coding,
    err_acquisition,
    err_convertion,
    err_memory,
    err_null_parameter,
    err_biometric_data,
    err_file,
    err_security,
    err_compression,
    err_FVP,    
    err_acquisition_status_unknown,
    err_acquisition_status_false_finger_detected,
    err_acquisition_status_moist_finger,
    err_acquisition_status_no_hit,
    err_FOTF_service_not_exist,
    err_certificate_already_exist,
    err_certificate_not_valid,
    err_certificate_private_key_not_available,
	err_database_size_exceed,
	err_sensor_fatal_error,
}

/**
 * A map that translates error code into string.
 *
 * 
 * <note>For product support, see <code>Sensor_error_code</code> enum</note>
 * 
 */
const map< Sensor_error_code, string > Sensor_error_msg_map =
{
        Sensor_error_code.err_unknown                               : "Unknown error",
}

/** The exception encompassing other sensor exceptions */
exception Sensor_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when sensor in use by other process */
exception Sensor_in_use_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when cancel Enrollment, Authentication or Identify any of this operations  */
exception Sensor_cancel_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when parameters received from client are not proper */
exception Sensor_parameter_mismatch_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor not connected or not found */
exception Sensor_device_not_found_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when process timed out */
exception Sensor_device_time_out_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor does not get power supply */
exception Sensor_power_supply_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when any error occurs in internal SDK */
exception Sensor_internal_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when ram database creation failed */
exception Sensor_database_creation_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when ram database try to store invalid template format  */
exception Sensor_invalid_template_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when command is not available */
exception Sensor_command_not_available_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when database record not available */
exception Sensor_user_record_not_available_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when try to store user with same user id available in database */
exception Sensor_user_exist_error    
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when sensor license is not installed in system */
exception Sensor_license_missing_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when coding error occurs in SDK */
exception Sensor_coding_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when sensor acquisition failed */
exception Sensor_acquisition_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when convertion failed */
exception Sensor_convertion_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor SDK memory problem occurs */
exception Sensor_memory_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when parameters sent is null */
exception Sensor_null_parameter_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when biometric data has error */
exception Sensor_biometric_data_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when can not access the log file */
exception Sensor_file_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when Security error occurs  */
exception Sensor_security_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when compression error occurs */
exception Sensor_compression_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when FVP error occured from SDK */
exception Sensor_FVP_error    
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor went to unknown status and could not be connected */
exception Sensor_acquisition_status_unknown_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor detects false finger */
exception Sensor_acquisition_status_false_finger_detected_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave sensor detects moist finger */
exception Sensor_acquisition_status_moist_finger_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when identification result is false */
exception Sensor_acquisition_status_no_hit_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when MorphoWave service not found in system */
exception Sensor_FOTF_service_not_exist_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when try load SSL certificate */
exception Sensor_certificate_already_exist_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when SSL certificate is not valid which user try to load */
exception Sensor_certificate_not_valid_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when SSL certificate dose not conatain private key */
exception Sensor_certificate_private_key_not_available_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when try to insert new user records and database size is already exceed limit */
exception Sensor_user_record_exceed_database_size_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when USB went into deadlock state */
exception Sensor_fatal_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}

/** The exception occurs when sensor upgrade version is not compatible with installed version */
exception Sensor_upgrade_incompatible_version_error
{
    1: required Sensor_error_code err_code;
    2: optional string what_UTF8;
}
